
#Introduction#

This package implements a new module nn.DAG which inherits from
nn.Container and allows to combine modules in an arbitrary graph
without cycle.

##Example##

A typical use would be:

```Lua
model = nn.DAG()

a = nn.Linear(100, 10)
b = nn.ReLU()
c = nn.Linear(10, 15)
d = nn.CMulTable()
e = nn.Linear(15, 15)

model:connect(a, b)
model:connect(b, nn.Linear(10, 15), nn.ReLU(), d)
model:connect(b, c)
model:connect(c, d)
model:connect(c, nn.Mul(-1), e)

model:setInput(a)
model:setOutput({ d, e })

input = torch.Tensor(30, 100):uniform()
output = model:updateOutput(input)

```

which would encode the following graph

                 +- Linear(10, 10) -> ReLU ---> d -->
                /                              /
               /                              /
    --> a --> b -----------> c --------------+
                              \
                               \
                                +-- Mul(-1) --> e -->

and run a forward pass with a random batch of 30 samples.

Note that DAG:connect allows to add a bunch of edges at once. This is
particularly useful to add anonymous modules which have a single
predecessor and successor.

##Input and output##

If a node has a single predecessor, its output is taken as-is. If it
has multiple predecessors, all the outputs are collected into a table,
and the table is used as input. The indexes of the outputs in that
table reflects the order in which the predecessors appeared in the
DAG:connect() commands.

The input to the DAG (respectively the produced output) is a nested
table of inputs reflecting the structure of the nested table of
modules provided to DAG:setInput (respectively DAG:setOutput)

So for instance, in the example above, the model expects a tensor as
input, since it is the input to the module a, and its output will is a
table composed of two tensors, corresponding to the outputs of d and e
respectively.

#Usage#

##nn.DAG()##

Create a new empty DAG, which inherits from nn.Container.

##nn.DAG:connect([module1 [, module2 [, ...]]])##

Add new nodes corresponding to the modules passed as arguments if they
are not already existing. Add edges between every two nodes
corresponding to a pair of successive modules in the arguments.

Calling it with n > 2 arguments is strictly equivalent to calling it
n-1 times on the pairs of successive arguments.

##nn.DAG:setInput(i)##

Defines the content and structure of the input. The argument should be
either a module, or a (nested) table of module. The input to the DAG
should be a (nested) table of inputs with the corresponding structure.

##nn.DAG:setOutput(o)##

Similar to DAG:setInput().

##nn.DAG:print()##

Prints the list of nodes.

##nn.DAG:saveDot(filename)##

Save a dot file to be used by the Graphviz set of tools for graph
visualization. This dot file can than be used for instance to produce
a pdf file with

```
dot graph.dot -T pdf -o graph.pdf
```

##nn.DAG:updateOutput(input)##

See the torch documentation.

##nn.DAG:updateGradInput(input, gradOutput)##

See the torch documentation.

##nn.DAG:accGradParameters(input, gradOutput, scale)##

See the torch documentation.

-- 
*Francois Fleuret, Jan 13th, 2017*
